@@ -164,6 +164,8 @@ class Pickler(object):
         self._max_iter = max_iter
         # Whether to allow decimals to pass-through
         self._use_decimal = use_decimal
+        # A cache of objects that have already been flattened.
+        self._flattened = {}
 
         if self.use_base85:
             self._bytes_tag = tags.B85
@@ -179,6 +181,7 @@ class Pickler(object):
         self._objs = {}
         self._depth = -1
         self._seen = []
+        self._flattened = {}
 
     def _push(self):
         """Steps down one level in the namespace."""
@@ -252,21 +255,29 @@ class Pickler(object):
         return self._flatten(obj)
 
     def _flatten(self, obj):
+        if self.unpicklable and self.make_refs:
+            result = self._flatten_impl(obj)
+        else:
+            try:
+                result = self._flattened[id(obj)]
+            except KeyError:
+                result = self._flattened[id(obj)] = self._flatten_impl(obj)
+        return result
 
+    def _flatten_impl(self, obj):
         #########################################
         # if obj is nonrecursive return immediately
         # for performance reasons we don't want to do recursive checks
         if PY2 and isinstance(obj, types.FileType):
             return self._flatten_file(obj)
 
-        if util.is_bytes(obj):
+        if type(obj) is bytes:
             return self._flatten_bytestring(obj)
 
-        if util.is_primitive(obj):
-            return obj
-
         # Decimal is a primitive when use_decimal is True
-        if self._use_decimal and isinstance(obj, decimal.Decimal):
+        if type(obj) in util.PRIMITIVES or (
+            self._use_decimal and isinstance(obj, decimal.Decimal)
+        ):
             return obj
         #########################################
 
@@ -308,37 +319,38 @@ class Pickler(object):
         return [self._flatten(v) for v in obj]
 
     def _get_flattener(self, obj):
-
-        list_recurse = self._list_recurse
-
-        if util.is_list(obj):
+        if type(obj) is list:
             if self._mkref(obj):
-                return list_recurse
+                return self._list_recurse
+            else:
+                self._push()
+                return self._getref
+
+        elif type(obj) is dict:
+            if self._mkref(obj):
+                return self._flatten_dict_obj
             else:
                 self._push()
                 return self._getref
 
         # We handle tuples and sets by encoding them in a "(tuple|set)dict"
-        if util.is_tuple(obj):
+        elif type(obj) is tuple:
             if not self.unpicklable:
-                return list_recurse
+                return self._list_recurse
             return lambda obj: {tags.TUPLE: [self._flatten(v) for v in obj]}
 
-        if util.is_set(obj):
+        elif type(obj) is set:
             if not self.unpicklable:
-                return list_recurse
+                return self._list_recurse
             return lambda obj: {tags.SET: [self._flatten(v) for v in obj]}
 
-        if util.is_dictionary(obj):
-            return self._flatten_dict_obj
-
-        if util.is_type(obj):
-            return _mktyperef
-
-        if util.is_object(obj):
+        elif util.is_object(obj):
             return self._ref_obj_instance
 
-        if util.is_module_function(obj):
+        elif util.is_type(obj):
+            return _mktyperef
+
+        elif util.is_module_function(obj):
             return self._flatten_function
 
         # instance methods, lambdas, old style classes...
@@ -721,9 +733,12 @@ class Pickler(object):
 
 
 def _in_cycle(obj, objs, max_reached, make_refs):
+    """Detect cyclic structures that would lead to infinite recursion"""
     return (
-        max_reached or (not make_refs and id(obj) in objs)
-    ) and not util.is_primitive(obj)
+        (max_reached or (not make_refs and id(obj) in objs))
+        and not util.is_primitive(obj)
+        and not util.is_enum(obj)
+    )
 
 
 def _mktyperef(obj):
